/*
 * productionMode.cpp
 *
 *  Created on: Jan 24, 2018
 *      Author: wn
 */


#include "defines.h"

#define MQTT_MAX_PACKET_SIZE 256


#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <PubSubClient.h>

#include "configuration.h"


typedef enum { NOT_PRESSED, PRESSED, LONG_PRESSED_PREPARE, LONG_PRESSED } tPressedState;




WiFiClientSecure espClient;
PubSubClient client(espClient);

extern uint32_t startTime;


void setup_wifi() {
	delay(10);
	WiFi.mode(WIFI_STA);

	// We start by connecting to a WiFi network
#ifdef DEBUG
	Serial.println();
	Serial.print("Connecting to ");
	Serial.println(configBlock.wifiSsid);
#endif

	WiFi.begin(configBlock.wifiSsid, configBlock.wifiKey);

	while (WiFi.status() != WL_CONNECTED) {
		delay(50);
#ifdef DEBUG
		Serial.print(".");
#endif
	}
	Serial.println("!");

#ifdef DEBUG
	Serial.println("");
	Serial.println("WiFi connected");
	Serial.println("IP address: ");
	Serial.println(WiFi.localIP());
#endif
}



void reconnect() {
	// Loop until we're reconnected
	while (!client.connected()) {
#ifdef DEBUG
		Serial.print("Attempting MQTT connection...");
#endif
		// Attempt to connect
		//char clientId[128];
		//snprintf(clientId, 127, "esp%s", WiFi.macAddress().c_str());
		if (client.connect(configBlock.mqttClientId, configBlock.mqttUser, configBlock.mqttPass)) {
#ifdef DEBUG
			Serial.println("connected");
#endif
			// Once connected, publish an announcement...
			client.publish(configBlock.mqttDebugTopic, "hello world");
		} else {
#ifdef DEBUG
			Serial.print("failed, rc=");
			Serial.print(client.state());
			Serial.println(" try again in 5 seconds");
#endif
			// Wait 5 seconds before retrying
			delay(5000);
		}
	}
}


void mqtt_connect() {
	if (!client.connected()) {
		reconnect();
	}
	client.loop();
}


void setupProduction() {
	setup_wifi();
	client.setServer(configBlock.mqttBroker, configBlock.mqttPort);
}


void loopProduction() {
	mqtt_connect();

	// discharge
	digitalWrite(TOUCH_PIN_SEND, LOW);
	pinMode(TOUCH_PIN_SEND, OUTPUT);
	digitalWrite(TOUCH_PIN_RECEIVE, LOW);
	pinMode(TOUCH_PIN_RECEIVE, OUTPUT);

	// sense
	pinMode(TOUCH_PIN_RECEIVE, INPUT);
	digitalWrite(TOUCH_PIN_SEND, HIGH);

	const uint32_t MAX_CHARGE_CNT = 1000;
	static uint32_t senseCnt = 0;
	static uint32_t chargeCntSum = 0;

	uint32_t chargeCnt = 0;

	noInterrupts();
	while ((! digitalRead(TOUCH_PIN_RECEIVE)) && (chargeCnt < MAX_CHARGE_CNT)) {
		chargeCnt++;
	}
	interrupts();

	chargeCntSum += chargeCnt;
	senseCnt++;

	static uint32_t lastMillis = 0;
	uint32_t currentMillis = millis();

	static uint32_t lastCalibration = 0;

	if (currentMillis - lastMillis > configBlock.period) {
		lastMillis = currentMillis;

		uint32_t chargeCntAvg = chargeCntSum / senseCnt;

		static uint32_t baseline = UINT32_MAX;
		if (chargeCntAvg < baseline) {
			baseline = chargeCntAvg;
		}

		uint32_t capNum = chargeCntAvg - baseline;

		if (configBlock.debugMode) {
			char payload[128];
			snprintf(payload, 127, "%ld %ld %ld %ld", senseCnt, baseline, chargeCntAvg, capNum);
			client.publish(configBlock.mqttDebugTopic, payload);
		}

		static tPressedState state = NOT_PRESSED;
		if (capNum > configBlock.threshold) {
#ifdef DEBUG
			Serial.println(capNum);
#endif

			switch (state) {
			case NOT_PRESSED:
				state = PRESSED;
				break;
			case PRESSED:
				state = LONG_PRESSED_PREPARE;
				break;
			case LONG_PRESSED_PREPARE:
				state = LONG_PRESSED;
				client.publish(configBlock.mqttTopic, "LONG_BEGIN");
				break;
			case LONG_PRESSED:
				state = LONG_PRESSED;
				client.publish(configBlock.mqttTopic, "LONG_CONT");
				break;
			}
		} else {
			if (currentMillis - lastCalibration > configBlock.calibrationTimeout) {
				baseline = UINT32_MAX;
				lastCalibration = currentMillis;
			}

			switch (state) {
			case NOT_PRESSED:
				break;
			case PRESSED:
			case LONG_PRESSED_PREPARE:
				client.publish(configBlock.mqttTopic, "SHORT");
				break;
			case LONG_PRESSED:
				client.publish(configBlock.mqttTopic, "LONG_END");
				break;
			}
			state = NOT_PRESSED;
		}

		chargeCntSum = 0;
		senseCnt = 0;
	}
}




